@@ -1,7 +1,15 @@
 /*
-   Time-stamp: <95/12/07 00:50:22 yusuf>
+   Time-stamp: <96/08/04 19:06:07 yusuf>
+
+   $Id: sel_backup.c,v 1.25 1996/08/04 18:08:37 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: sel_backup.c,v 1.25 1996/08/04 18:08:37 yusuf Exp $";
+#endif /* lint */
+
+
 
 #include "taper.h"
 #include "backup.h"
@@ -9,10 +17,10 @@
 select_details  file_sd, select_sd;
 int dir_left_width;
 
-void backup_asterix_line(WINDOW *win, long ent_num, int line)
+void backup_asterix_line(WINDOW *win, _s32 ent_num, int line)
 {
     struct direntry *entry;
-    char       s2[MAXNAMLEN];
+    char       s2[MAX_FNAME];
     struct oa_file_entry *fe;
     char   c1;
     int    c;
@@ -33,22 +41,23 @@ void backup_asterix_line(WINDOW *win, long ent_num, int line)
 	    break;
 	}
 	if (strlen(name(fe)) <= strlen(s2))	 /* check that whole directory not selected */
-	  if (!strncmp(s2, name(fe), strlen(name(fe)))) {
+	  if (!strncmp(s2, name(fe), strlen(name(fe)))) 
+	    if (S_ISDIR(fe->i.mode)) {		 /* only if a directory */
 	      c1 = '*';
 	      break;
 	  }
 	advance_ofe(&fe);
     }
-    if (exclude_archive(s2, entry->info.st_mode))
+    if (exclude_archive(s2))
       c1 = ' ';
     mvwaddch(files, line+1, left_width-3, c1);
 }
 
 
-void backup_asterix_dir(WINDOW *win, long start, char *p_scroll) {
+void backup_asterix_dir(WINDOW *win, _s32 start, char *p_scroll) {
 /* Prints an asterix next to selected entries in the directory */    
     int        cur_line=1;
-    long       cur_ent;
+    _s32       cur_ent;
     
     cur_ent=start;
     while (cur_line < screen_ylen_files-3) {
@@ -60,7 +69,7 @@ void backup_asterix_dir(WINDOW *win, long start, char *p_scroll) {
 }
 
 
-void backup_print_dir_line(WINDOW *win, long ent_num, int line, char ref) {
+void backup_print_dir_line(WINDOW *win, _s32 ent_num, int line, char ref) {
 /* Prints one entry in a directory at line 'line'  
  * Returns the size of the directory entry in bytes */
     
@@ -73,7 +82,7 @@ void backup_print_dir_line(WINDOW *win, long ent_num, int line, char ref) {
 }
 
 
-void backup_print_dir(WINDOW *win, long start, char *p_scroll) {
+void backup_print_dir(WINDOW *win, _s32 start, char *p_scroll) {
 /* This function prints the current directory into the left hand
  * window. It starts printing the directory at entry 'start'.
  * 
@@ -86,9 +95,9 @@ void backup_print_dir(WINDOW *win, long start, char *p_scroll) {
  * where * = ' ', '/' or '@' depending on file type 
 */
     int        cur_line=1;
-    long       cur_ent;
-    long       dsize=0;
-    char       s[MAXNAMLEN];
+    _s32       cur_ent;
+    _s32       dsize=0;
+    char       s[MAX_FNAME];
     struct direntry *entry;
     
     my_werase(win, COLOR_DIRECTORY);
@@ -122,7 +131,7 @@ void backup_print_dir(WINDOW *win, long start, char *p_scroll) {
     wattroff(win, A_UNDERLINE);
 }
 
-void backup_print_selected_line(WINDOW *win, long entry, int line, char ref) {
+void backup_print_selected_line(WINDOW *win, _s32 entry, int line, char ref) {
     
     struct oa_file_entry *s;
     char   s1[50], s2[200];
@@ -130,11 +139,13 @@ void backup_print_selected_line(WINDOW *win, long entry, int line, char ref) {
     s = find_entry(sel_files, entry);
     switch(s->selected) {
        case 1:
-	sprintf(s2, "F   %9s %s ", print_kb(s1, s->i.size), name(s));
+	(s->i.size < 1024) ? sprintf(s2, "F   %8d  %s ", s->i.size, name(s)) :
+	   sprintf(s2, "F   %9s %s ", print_kb(s1, s->i.size), name(s));
 	break;
        case 2:
        case 3:
-	sprintf(s2, "F ( %9s %s)", print_kb(s1, s->i.size), name(s));
+	(s->i.size < 1024) ? sprintf(s2, "F ( %8d  %s)", s->i.size, name(s)) :
+  	  sprintf(s2, "F ( %9s %s)", print_kb(s1, s->i.size), name(s));
 	break;
     }
     if (s->incremental) *s2 = 'I';
@@ -155,8 +166,8 @@ void backup_print_selected_size(WINDOW *win)
     wattroff(win, A_UNDERLINE);
 }
 
-void backup_print_selected(WINDOW *win, long top_sel, char *p_scroll) {
-    long   cur;
+void backup_print_selected(WINDOW *win, _s32 top_sel, char *p_scroll) {
+    _s32   cur;
 
     my_werase(win, COLOR_SELECTED);
     backup_print_selected_size(win);
@@ -172,13 +183,13 @@ void backup_print_selected(WINDOW *win, long top_sel, char *p_scroll) {
 }
 
 
-long sz;
+_s32 sz;
 _errstat sel_backup_dpd(char *full_path, struct stat *b)
 {
     sz += b->st_size;
     return 0;
 }
-long get_dirsize(WINDOW *mes, int ln, char *full_dir, char inc)
+_s32 get_dirsize(WINDOW *mes, int ln, char *full_dir, char inc)
 {
 /* gets the size of the directory and all it's contents
    if inc == 1, then incremental size is used
@@ -187,42 +198,46 @@ long get_dirsize(WINDOW *mes, int ln, char *full_dir, char inc)
 */
 
     sz = 0;
-    process_dir(mes, ln, full_dir, inc, sel_backup_dpd, FALSE);
+    if (process_dir(mes, ln, full_dir, inc, sel_backup_dpd, FALSE) == -1)
+      return -1;
     return sz;
 }
 
 
-void recalculate_size(struct oa_file_entry *ce, struct file_info *fi, int printbox)
+_errstat recalculate_size(struct oa_file_entry *ce, struct file_info *fi, int printbox)
 {
-    long old_size;
+/* Returns -1 if user aborted the recalc */    
+    _s32 old_size, sz;
     WINDOW *s_box=NULL;
-    char   s[MAXNAMLEN];
+    char   s[MAX_FNAME];
     
     old_size = ce->i.size;
     strcpy(s, "Sizing "); strcat(s, name(ce));
     if (printbox && !no_windows) s_box=status_box(s_box, s, 1, TRUE, 1);
-    if (S_ISDIR(fi->mode))
-      ce->i.size = get_dirsize(s_box, 1, name(ce), ce->incremental);
-    else
-      ce->i.size = (file_more_recent(name(ce), NULL)) ? fi->size : 0;
-    total_selected += ce->i.size-old_size;
+    sz = get_dirsize(s_box, 1, name(ce), ce->incremental);   
     if (printbox) close_statusbox(s_box);
+    if (sz != -1) { 				 /* aborted directory sizing */
+	ce->i.size = sz;
+	total_selected += ce->i.size-old_size;
+	return 0;
+    }
+    return -1;
 }
 
 
 void select_entry_engine(struct file_info *fi, struct oa_file_entry *ce, 
 			 char *fn, int printbox, int add, int sel_method)
 {
-    long   c;
+    _s32   c;
     struct oa_file_entry *se;
     
     if (add) {					 /* new file */
 	no_sel++;
+	ce->i.name_len = make4len(fn);
 	add_sel_file(ce, fn);
 	ce = find_entry(sel_files, no_sel-1);
 	ce->i.size = 0;
 	ce->incremental = incremental;
-	ce->i.name_len = strlen(fn);
 	strcpy(name(ce), fn);
     }
     ce->i.act_size = 0;
@@ -235,7 +250,10 @@ void select_entry_engine(struct file_info *fi, struct oa_file_entry *ce,
     ce->i.ctime = fi->ctime;
     ce->i.mtime  = fi->mtime;
     ce->selected = sel_method;
-    recalculate_size(ce, fi, printbox);
+    if (recalculate_size(ce, fi, printbox) == -1) {
+	if (add) no_sel--;
+	return;
+    }
     if (!S_ISDIR(ce->i.mode)) {
 	se = sel_files;				 /* make sure that this */
 	for (c=0; c<no_sel; c++) {		 /* file is not going to  */
@@ -275,7 +293,7 @@ int select_entry_1(struct direntry *entry, char *s)
  * Returns 0 if entry existed */
     struct file_info fi;
     struct oa_file_entry *ce;
-    long   c=0;
+    _s32   c=0;
 	  
     ce = sel_files;
     while (c < no_sel)  {
@@ -286,7 +304,7 @@ int select_entry_1(struct direntry *entry, char *s)
 	c++;
 	advance_ofe(&ce);
     }
-    if (exclude_archive(s, entry->info.st_mode)) {
+    if (exclude_archive(s)) {
 	bmessage_box("This file is on the file exclusion list", MB_OK);
 	return 0;
     }
@@ -300,12 +318,13 @@ int select_entry_1(struct direntry *entry, char *s)
     fi.atime = entry->info.st_atime;
     fi.ctime = entry->info.st_ctime;
     fi.mtime  = entry->info.st_mtime;
+    fi.name_len = make4len(s);
     select_entry_engine(&fi, ce, s, 1, c==no_sel, 1);
     return 1;
 }
 
 
-int select_entry(long cur_entry) {
+int select_entry(_s32 cur_entry) {
 /* Adds the 'cur_entry' entry into the select files. If a directory,
  * confirms with user that the whole subdirectory tree is to be
  * included
@@ -313,7 +332,7 @@ int select_entry(long cur_entry) {
  * Returns 1 if entry added, 0 otherwise
 */
     struct direntry  *entry;
-    char   s[MAXNAMLEN];
+    char   s[MAX_FNAME];
 
     entry = directory + cur_entry;
     if (!strcmp(entry->entry.d_name, "..")) {
@@ -329,17 +348,18 @@ int select_entry(long cur_entry) {
       strcat(s, "/"); 
     strcat(s, entry->entry.d_name);
     if (S_ISDIR(entry->info.st_mode))
-      strcat(s, "/");
+      if (s[strlen(s)-1] != '/')
+	strcat(s, "/"); 
     return select_entry_1(entry, s);
 }
 
 
-void backup_update_selected()
+void backup_update_selected(void)
 {
 /* Goes through archive and works out indirect selections 
    for sel_backup */
 
-    long c, c1;
+    _s32 c, c1;
     struct oa_file_entry *cur_ent, *se;
 
     cur_ent = sel_files;			 /* clear current selection */
@@ -381,7 +401,7 @@ void backup_update_selected()
 }
 
 
-int backup_toggle_selected(long cur_entry)
+int backup_toggle_selected(_s32 cur_entry)
 {
 /* Toggles selection between incremental & non-incremental mode */
 
@@ -389,7 +409,8 @@ int backup_toggle_selected(long cur_entry)
 
     ce = find_entry(sel_files,cur_entry);
     ce->incremental = !ce->incremental;
-    recalculate_size(ce, &ce->i, 1);
+    if (recalculate_size(ce, &ce->i, 1) == -1)
+      ce->incremental = !ce->incremental;	 /* user aborted - restore */
     backup_print_selected_size(selection);
     return 0;
 }
@@ -399,10 +420,10 @@ void del_sel(struct oa_file_entry *fe)
 /* Delete the entry fe from sel_files */
 {
     char *dest, *src;
-    long bytes_to_go, len;
+    _s32 bytes_to_go, len;
     
     dest = (char *) fe;				 /* now delete the entry */
-    len = (memory_tight) ? sizeof(struct oa_file_entry) + strlen(name(fe))+1   /* work out how many bytes to delete */
+    len = (memory_tight) ? sizeof(struct oa_file_entry) + fe->i.name_len   /* work out how many bytes to delete */
       : sizeof(struct fixed_oa);
     src = dest+len;				 
     bytes_to_go = len_sel_files - (src-(char *) sel_files);/* how many bytes to copy */
@@ -413,7 +434,7 @@ void del_sel(struct oa_file_entry *fe)
     len_sel_files -= len;
 }
 
-void backup_del_cur_entry_engine(long entry)
+void backup_del_cur_entry_engine(_s32 entry)
 {
     umode_t  mode;
     struct oa_file_entry *fe;
@@ -428,7 +449,7 @@ void backup_del_cur_entry_engine(long entry)
     backup_print_selected_size(selection);
 }
     
-void backup_del_cur_entry(select_details *sd, long *no_sel)
+void backup_del_cur_entry(select_details *sd, _s32 *no_sel)
 {
     backup_del_cur_entry_engine(sd->cur_entry);
 }
@@ -436,20 +457,20 @@ void backup_del_cur_entry(select_details *sd, long *no_sel)
 
 
 
-void backup_save_backupset()
+void backup_save_backupset(int in_backup)
 {
     backrest_save_backupset(1);
 }
   
 
-void backup_restore_backupset()
+void backup_restore_backupset(void)
 {
-    char ln[MAXNAMLEN];
+    char ln[MAX_FNAME];
     FILE *f;
     struct oa_file_entry *ce;
     struct direntry entry;
-    long c;
-    char x[MAXNAMLEN];
+    _s32 c;
+    char x[MAX_FNAME];
     char *x1;
     int x2;
 
@@ -486,13 +507,13 @@ void backup_restore_backupset()
     fclose(f);
 }
 
-int backup_select_vol(long entry)
+int backup_select_vol(_s32 entry)
 {
 /* Tag an entry in the volume window */    
-    long c=0;
+    _s32 c=0;
     struct direntry c_entry;
     char *s;
-    char s1[MAXNAMLEN];
+    char s1[MAX_FNAME];
     
     s = vol_details;				 /* find appropriate */
     while (c++<entry)				 /* line in vol_details buffer */
@@ -513,10 +534,10 @@ int backup_select_vol(long entry)
     return 0;
 }
 
-void backup_unselect_dir(select_details *sd, long *dummy)
+void backup_unselect_dir(select_details *sd, _s32 *dummy)
 {
     struct direntry *entry;
-    char       s2[MAXNAMLEN];
+    char       s2[MAX_FNAME];
     struct oa_file_entry *fe;
     char   c1=' ';
     int    c;
@@ -543,7 +564,7 @@ void backup_unselect_dir(select_details *sd, long *dummy)
 	advance_ofe(&fe);
     }
     if (!S_ISDIR(entry->info.st_mode)) /* see if it will be excluded */
-      if (exclude_archive(s2, entry->info.st_mode))
+      if (exclude_archive(s2))
         c1 = ' ';
     switch(c1) {
      case ' ':
@@ -560,14 +581,44 @@ void backup_unselect_dir(select_details *sd, long *dummy)
 
 
 
-    
+
+void add_root(struct direntry **dir, _s32 *dircount)
+{
+/*  Makes sure that a '..' directory entry exists and if it
+    doesn't, adds it
+
+    Returns nothing
+
+*/
+  _s32 c=0;
+  struct direntry *d;
+
+
+  d = *dir;
+  while (c<*dircount) {
+    if (!strcmp(d->entry.d_name, "..")) return;	 /* OK  */
+    c++;
+    d++;
+  }
+  /* no .. entry - add one */
+  
+  *dir = my_realloc((void *) *dir, sizeof(struct direntry)    
+		    * ((*dircount)+1));		 /* make memory */
+  for (c=*dircount-1; c>=0; c--)		 /* add this entry at top */
+    *(*dir+c+1) = *(*dir+c);
+  strcpy((*dir)->entry.d_name, "..");		 /* add the .. entry */
+  (*dir)->org_info.st_mode = S_IFDIR;
+  (*dircount)++;
+}
+
+
 int backup_select_files(char *cur_dir) {
 /* Returns 0 if quit
  * Returns 1 if wants to backup
 */
     int   ret;
     char  p=0;
-    char  old_dir[MAXNAMLEN];
+    char  old_dir[MAX_FNAME];
 
     dir_left_width = left_width;		 /* for directory routines */
     dir_screen_ylen = screen_ylen_files;
@@ -636,21 +687,33 @@ int backup_select_files(char *cur_dir) {
 	 case SELECT_ENTER:
 	    if (S_ISDIR((dir_sd.cur_entry+directory)->org_info.st_mode)) {
 		*old_dir = 0;
-		if (!strcmp((directory+dir_sd.cur_entry)->entry.d_name, ".."))
-		  strcpy(old_dir, cur_dir);
+		if (!strcmp((directory+dir_sd.cur_entry)->entry.d_name, "..")) {
+		  if (strcmp(cur_dir, "/"))
+		      strcpy(old_dir, cur_dir);
+		  else {			 /* top dir only */
+		    directory = my_realloc(directory, sizeof(struct direntry));
+		    strcpy(directory->entry.d_name, ""); /* add the .. entry */
+		    directory->org_info.st_mode = S_IFDIR;
+		    directory_count = 1;
+		    strcpy(cur_dir, ""); strcpy(old_dir, "..");
+		    goto madedir;
+		  }
+		}
 		strcat(cur_dir, "/");
 		strcat(cur_dir, (directory+dir_sd.cur_entry)->entry.d_name);
 		ret = read_dir(cur_dir, "", NULL,
 			       &directory, &directory_count);
+		add_root(&directory, &directory_count);
 		if (ret == -1) return -1;
 		if (ret == 0)
-		  getcwd(cur_dir, MAXNAMLEN);		
+		  getcwd(cur_dir, MAX_FNAME);		
 		else {
 		    clear_sd(&dir_sd);
 		    if (old_dir)
 		      find_correct_sd(old_dir,&dir_sd, directory_count,
 				      directory);
 		}
+	    madedir:;
 		p=0;
 		backup_print_dir(files, dir_sd.top_entry, &p);
 		p=0;
@@ -661,4 +724,5 @@ int backup_select_files(char *cur_dir) {
 	    break;
 	}
     }
+    return 0; /* never get here */
 }
