@@ -1,21 +1,16 @@
 /*
-   Time-stamp: <95/12/07 00:50:45 yusuf>
+   Time-stamp: <96/07/19 20:18:22 yusuf>
+
+   $Id: select_box.c,v 1.19 1996/07/27 20:42:13 yusuf Exp $	
+
 */
 
+#ifndef lint
+static char vcid[] = "$Id: select_box.c,v 1.19 1996/07/27 20:42:13 yusuf Exp $";
+#endif /* lint */
 
-#define inline __inline__
 
-#include <dirent.h>
-#include <errno.h>
-#include <unistd.h>
-#include <stdlib.h>
-#include <curses.h>
-#include <form.h>
-#include <fcntl.h>
-#include <sys/stat.h>
 
-#include "memory.h"
-#include "errors.h"
 #include "select_box.h"
 
 
@@ -23,12 +18,12 @@ extern WINDOW *win_main;
 WINDOW *dir_win;
 struct direntry      *sb_directory;		 /* for the select directory */
 struct direntry *directory;
-long sb_directory_count;			 /* routines */
-long directory_count;
+_s32 sb_directory_count;			 /* routines */
+_s32 directory_count;
 select_details sb_dir_sd;
 select_details dir_sd;
 char *global_cur_dir;
-char  dir_cur_dir[MAXNAMLEN];
+char  dir_cur_dir[MAX_FNAME];
 int sb_dir_left_width;
 int dir_screen_ylen;
 
@@ -42,7 +37,7 @@ void my_init_windows()
 }
 
 
-inline void my_strcpy(char *dest, char *src, int maxlen)
+void my_strcpy(char *dest, char *src, int maxlen)
 {
 /* Copies string src to dest but up to a maximum of maxlen chars 
  * Also removes trailing spaces
@@ -113,12 +108,12 @@ void my_werase(WINDOW *win, int color)
 }
 
 
-inline long max(long a, long b) {
+_s32 max(_s32 a, _s32 b) {
     return (a > b) ? a : b;
 }
 
 
-inline long min(long a, long b) {
+_s32 min(_s32 a, _s32 b) {
     return (a < b) ? a : b;
 }
 
@@ -176,6 +171,7 @@ int get_string(WINDOW *win, char *s, int maxlen, char *prompt)
 	}
 	if (c == '\r' || c == '\n' || c == KEY_F0+10) break;
 	form_driver(form, c);
+	wrefresh(form->w);
     }
     form_driver(form, REQ_VALIDATION);
     my_strcpy(s, field_buffer(field[0], 0), maxlen); 
@@ -258,6 +254,79 @@ void close_statusbox(WINDOW *win) {
 }
 
 
+void mb_options(WINDOW *mes_box, int mes_box_lines, int type, int res, int bold)
+{
+/* Prints the options for a message box.
+ * 
+ * If bold==1, then it's printed bold, otherwise it's inverse */
+    
+    switch(type) {
+     case MB_RETRYABORT:
+	if (res) {
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, 3, "RETRY");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 7, "ABORT");
+	}
+	else {
+	    mvwaddstr(mes_box, mes_box_lines- 1, 3, "RETRY");
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 7, "ABORT");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	}
+	break;
+     case MB_YESNO:
+	if (res) {
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, 3, "YES");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 5, "NO");
+	}
+	else {
+	    mvwaddstr(mes_box, mes_box_lines- 1, 3, "YES");
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 5, "NO");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	}
+	break;
+     case MB_APPENDOVERWRITE:
+	if (res) {
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, 3, "APPEND");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 12, "OVERWRITE");
+	}
+	else {
+	    mvwaddstr(mes_box, mes_box_lines- 1, 3, "APPEND");
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 12, "OVERWRITE");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	}
+	break;
+     case MB_OKCANCEL:
+	if (res) {
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, 3, "OK");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 9, "CANCEL");
+	}
+	else {
+	    mvwaddstr(mes_box, mes_box_lines- 1, 3, "OK");
+	    wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	    mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 9, "CANCEL");
+	    wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	}
+	break;
+     case MB_OK:
+	wattron(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx/2-1, "OK");
+	wattroff(mes_box, ((bold) ? A_BOLD : A_REVERSE));
+	break;
+    }
+    wrefresh(mes_box);
+}
+
+
 int message_box_engine(WINDOW *mes_box, int type) {
 /* Prints a message box with the message 's'. The user has the option of
  * answering YES or NO. The box is printed in the middle of the screen
@@ -271,86 +340,24 @@ int message_box_engine(WINDOW *mes_box, int type) {
     int     mes_box_lines=mes_box->_maxy;
 
     while (1) {
-      switch(type) {
-       case MB_RETRYCANCEL:
-	if (res) {
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, 3, "RETRY");
-	  wattroff(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 7, "CANCEL");
-	}
-	else {
-	  mvwaddstr(mes_box, mes_box_lines- 1, 3, "RETRY");
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 7, "ABORT");
-	  wattroff(mes_box, A_REVERSE);
-	}
-	break;
-       case MB_YESNO:
-	if (res) {
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, 3, "YES");
-	  wattroff(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 5, "NO");
-	}
-	else {
-	  mvwaddstr(mes_box, mes_box_lines- 1, 3, "YES");
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 5, "NO");
-	  wattroff(mes_box, A_REVERSE);
-	}
-	break;
-      case MB_APPENDOVERWRITE:
-	if (res) {
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, 3, "APPEND");
-	  wattroff(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 12, "OVERWRITE");
-	}
-	else {
-	  mvwaddstr(mes_box, mes_box_lines- 1, 3, "APPEND");
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 12, "OVERWRITE");
-	  wattroff(mes_box, A_REVERSE);
-	}
-	break;
-      case MB_OKCANCEL:
-	if (res) {
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, 3, "OK");
-	  wattroff(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 9, "CANCEL");
-	}
-	else {
-	  mvwaddstr(mes_box, mes_box_lines- 1, 3, "OK");
-	  wattron(mes_box, A_REVERSE);
-	  mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx - 9, "CANCEL");
-	  wattroff(mes_box, A_REVERSE);
-	}
-	break;
-      case MB_OK:
-	wattron(mes_box, A_REVERSE);
-	mvwaddstr(mes_box, mes_box_lines-1, mes_box->_maxx/2-1, "OK");
-	wattroff(mes_box, A_REVERSE);
-	break;
-      }
-      wrefresh(mes_box);
+	mb_options(mes_box, mes_box_lines, type, res, 0);
 	c = wgetch(mes_box);
 	switch(c) {
-      case 'y':
-      case 'Y':
-	res = 1; break;
-      case 'n':
-      case 'N':
-	res = 0; break;
-      case ' ':
-      case '\t':
-      case KEY_LEFT:
-      case KEY_RIGHT:
-	res = !res; break;
-      }
-      if ((c == '\n') || (c=='\r')) break;
+	 case 'y':
+	 case 'Y':
+	    res = 1; break;
+	 case 'n':
+	 case 'N':
+	    res = 0; break;
+	 case ' ':
+	 case '\t':
+	 case KEY_LEFT:
+	 case KEY_RIGHT:
+	    res = !res; break;
+	}
+	if ((c == '\n') || (c=='\r')) break;
     }
+    mb_options(mes_box, mes_box_lines, type, res, 1);
     my_delwin(mes_box);
     return res;
 }
@@ -414,14 +421,14 @@ int multi_message_box(char lns[][150], int count, int type) {
 }
 
     
-inline void clear_sd(select_details *sd) {
+void clear_sd(select_details *sd) {
     sd->top_entry = 0;
     sd->cur_entry = 0;
     sd->cursor_line = 1;
 }
 
 
-void check_eop(WINDOW *win, select_details *sd, long no_entries, print_screen ps)
+void check_eop(WINDOW *win, select_details *sd, _s32 no_entries, print_screen ps)
 {
     char p=0;
     
@@ -440,7 +447,7 @@ void check_eop(WINDOW *win, select_details *sd, long no_entries, print_screen ps
     
 
     
-void print_scroll_bar(WINDOW *win, long no_entries, select_details *sd,
+void print_scroll_bar(WINDOW *win, _s32 no_entries, select_details *sd,
 		      int s_len, int s_width, char *p_scroll)
 {
     int c;
@@ -462,16 +469,16 @@ void print_scroll_bar(WINDOW *win, long no_entries, select_details *sd,
 }
 
     
-int select_box(WINDOW *win, long *no_entries,  select_details *sd,
+int select_box(WINDOW *win, _s32 *no_entries,  select_details *sd,
 	       print_screen ps, print_line pl, tag_entry te, delete_entry de,
 	       refresh_screen rs,
-	       WINDOW *sel_win, long *sel_no_entries,
+	       WINDOW *sel_win, _s32 *sel_no_entries,
 	       select_details *sel_sd, print_screen sel_ps,
 	       print_screen asterix, select_details *filed,
 	       char *select_help, sr_backupset save_backupset,
-	       sr_backupset rest_backupset) 
+	       sr_restoreset rest_backupset) 
 {
-    long  c;
+    _s32  c;
     int   screen_len, screen_width, teret;
     char  p_scroll=0, dummy;
     WINDOW *help;
@@ -495,7 +502,7 @@ int select_box(WINDOW *win, long *no_entries,  select_details *sd,
 	 case 'B':
 	 case 'b':
 	    if (save_backupset) {
-		save_backupset(); 
+		save_backupset(0); 
 		rs();
 		if (sel_ps) sel_ps(sel_win, sel_sd->top_entry, &p_scroll);
 	    }
@@ -649,6 +656,7 @@ int select_box(WINDOW *win, long *no_entries,  select_details *sd,
 	    sd->cursor_line = sd->cur_entry - sd->top_entry + 1;
 	}
     }
+    return 0; /* shouldn't get here */
 }
 
 
@@ -710,12 +718,13 @@ int select_menu(WINDOW *win, char *menu_items[], int *in_op)
     if (cur_line < 0)
       cur_line = no_items;
   }
+  return 0; /* shouldn't get here */
 }
 
 
 
 
-char *convert(char *s, long num)
+char *convert(char *s, _s32 num)
 {
 /* prints a long number in the form 123,456,789  */
     
@@ -747,7 +756,7 @@ int comp(const void *da1, const void *da2) {
 
 
 int read_dir(char *dir_name, char *prefix, modify_filename fn, 
-	     struct direntry **den, long *den_count) {
+	     struct direntry **den, _s32 *den_count) {
 /* Reads the directory pointed to by 'dir' into memory.
  *
  *  Only selects files that are prefixed with 'prefix'.
@@ -765,7 +774,7 @@ int read_dir(char *dir_name, char *prefix, modify_filename fn,
     DIR            *dir_ptr;
     struct direntry *cur_ent;
     struct dirent  *x;
-    long            count=0, in_count;
+    _s32            count=0, in_count;
 
     if ((dir_ptr = opendir(dir_name)) == NULL)	{ /* open directory */
 	switch(errno) {
@@ -778,7 +787,7 @@ int read_dir(char *dir_name, char *prefix, modify_filename fn,
 	return 0;
     }
     chdir(dir_name);
-    getcwd(dir_name, MAXNAMLEN);		 /* real name */	
+    getcwd(dir_name, MAX_FNAME);		 /* real name */	
     *den_count = 0;				 /* count entries in dir */
     while (readdir(dir_ptr)) (*den_count)++; 
     closedir(dir_ptr);
@@ -845,7 +854,7 @@ void print_dir_line(WINDOW *win, struct direntry *entry, int line,
  * Returns the size of the directory entry in bytes */
     
 
-    char       s[NAME_MAX+1], s2[MAXNAMLEN];
+    char       s[NAME_MAX+1], s2[MAX_FNAME];
 
     get_file_type(s, entry->info.st_mode);
     strcat(s, " "); 
@@ -870,13 +879,13 @@ void print_dir_line(WINDOW *win, struct direntry *entry, int line,
 }
 
 
-void sb_print_dir_line(WINDOW *win, long ent_num, int line, char ref)
+void sb_print_dir_line(WINDOW *win, _s32 ent_num, int line, char ref)
 {
     print_dir_line(win, sb_directory+ent_num, line, ref, sb_dir_left_width);
 }
     
 
-void print_dir(WINDOW *win, long start, char *p_scroll) {
+void print_dir(WINDOW *win, _s32 start, char *p_scroll) {
 /* This function prints the current directory into the left hand
  * window. It starts printing the directory at entry 'start'.
  * 
@@ -889,9 +898,9 @@ void print_dir(WINDOW *win, long start, char *p_scroll) {
  * where * = ' ', '/' or '@' depending on file type 
 */
     int        cur_line=1;
-    long       cur_ent;
-    long       dsize=0;
-    char       s[MAXNAMLEN];
+    _s32       cur_ent;
+    _s32       dsize=0;
+    char       s[MAX_FNAME];
     struct direntry *entry;
     
     my_werase(win, COLOR_DIRECTORY);
@@ -925,17 +934,17 @@ void print_dir(WINDOW *win, long start, char *p_scroll) {
     wattroff(win, A_UNDERLINE);
 }
 
-void dir_paint()
+void dir_paint(void)
 {
     touchwin(dir_win); wrefresh(dir_win);
 }
 
-void set_correct_sd(char *s, select_details *sd, long dir_count, 
+void set_correct_sd(char *s, select_details *sd, _s32 dir_count, 
 		    struct direntry *dir)
 {
 /* Sets the default entry as 's' in directory */
     
-    long c=0;
+    _s32 c=0;
     
     while (c < dir_count)
       if (!strcmp(s, (dir+c)->entry.d_name))
@@ -954,7 +963,7 @@ void set_correct_sd(char *s, select_details *sd, long dir_count,
 }
 
 
-void find_correct_sd(char *old_dir, select_details *sd, long dir_count,
+void find_correct_sd(char *old_dir, select_details *sd, _s32 dir_count,
 		     struct direntry *dir)
 {
 /* Used when going up a directory. Finds the entry in the current directory
@@ -991,7 +1000,7 @@ int select_file(char *start_dir, char *prefix, char *deflt, modify_filename fn,
     
     int   ret;
     char  p=0;
-    char  old_dir[MAXNAMLEN], cur_dir[MAXNAMLEN];
+    char  old_dir[MAX_FNAME], cur_dir[MAX_FNAME];
 
     getcwd(cur_dir, sizeof(cur_dir));		 /* preserve existing dir */
     strcpy(dir_cur_dir, start_dir);		 /* don't alter start_dir */
@@ -1047,7 +1056,7 @@ int select_file(char *start_dir, char *prefix, char *deflt, modify_filename fn,
 			return -1;
 		    }
 		    if (ret == 0)
-		      getcwd(dir_cur_dir, MAXNAMLEN);
+		      getcwd(dir_cur_dir, MAX_FNAME);
 		    else
 		      clear_sd(&sb_dir_sd);
 		    if (old_dir)
@@ -1070,4 +1079,23 @@ int select_file(char *start_dir, char *prefix, char *deflt, modify_filename fn,
 	    break;
 	}
     }
+    return 0;  /* never gets here */
+}
+
+
+
+int retryabort(char *s)
+{
+/* Prints an error message and then offers user the option
+ * of retrying or aborting
+ * 
+ * Returns 0 if retry,
+ *        -1 if abort.
+*/
+    char s1[MAX_FNAME];
+    
+    strcpy(s1, strerror(errno));
+    strcat(s1, " while ");
+    strcat(s1, s);
+    return (message_box(s1, MB_RETRYABORT) == 1) ? 0 : -1;
 }
